import globalPluginHandler
import gui
import wx
import core
import urllib.request
import json
import locale
import config
import addonHandler
from scriptHandler import script

# Dodajemy magiczne "summary" z manifestu, które robi kategorię skrótów
ADDON_SUMMARY = addonHandler.getCodeAddon().manifest["summary"]

locale.setlocale(locale.LC_ALL, 'pl_PL.UTF-8')

confspec = {
    "defaultCity": "string(default='')"
}
config.conf.spec["polskapogoda"] = confspec

class PolskaPogodaSettingsPanel(gui.settingsDialogs.SettingsPanel):
    title = "PolskaPogoda"

    def makeSettings(self, sizer):
        self.cityBox = wx.Choice(self)
        sizer.Add(wx.StaticText(self, label="Domyślne miasto do szybkiego odczytu:"), 0, wx.ALL, 5)
        sizer.Add(self.cityBox, 0, wx.ALL | wx.EXPAND, 5)

        try:
            url = "https://danepubliczne.imgw.pl/api/data/synop"
            response = urllib.request.urlopen(url)
            data = json.loads(response.read())
            filtered = [d for d in data if "platforma" not in d["stacja"].lower()]
            cityNames = sorted([d["stacja"] for d in filtered], key=locale.strxfrm)

            self.cityBox.AppendItems(cityNames)
            savedCity = config.conf["polskapogoda"]["defaultCity"]
            if savedCity in cityNames:
                self.cityBox.SetStringSelection(savedCity)
        except:
            pass

    def onSave(self):
        city = self.cityBox.GetStringSelection()
        if city:
            config.conf["polskapogoda"]["defaultCity"] = city

class WeatherResultDialog(wx.Dialog):
    def __init__(self, weatherText):
        super().__init__(None, size=(400, 300))
        panel = wx.Panel(self)
        sizer = wx.BoxSizer(wx.VERTICAL)

        resultBox = wx.TextCtrl(panel, value=weatherText, style=wx.TE_MULTILINE | wx.TE_READONLY | wx.BORDER_NONE)
        resultBox.SetBackgroundColour(panel.GetBackgroundColour())
        resultBox.SetFont(wx.Font(11, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL))

        okButton = wx.Button(panel, wx.ID_OK, label="OK")
        okButton.Bind(wx.EVT_BUTTON, lambda evt: self.Close())

        sizer.Add(resultBox, 1, wx.ALL | wx.EXPAND, 15)
        sizer.Add(okButton, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        panel.SetSizer(sizer)

        accel_tbl = wx.AcceleratorTable([
            (wx.ACCEL_NORMAL, wx.WXK_ESCAPE, wx.ID_CANCEL),
            (wx.ACCEL_NORMAL, wx.WXK_RETURN, wx.ID_OK),
        ])
        self.SetAcceleratorTable(accel_tbl)

        self.Bind(wx.EVT_CLOSE, self.onClose)
        self.Centre()
        self.Show()

    def onClose(self, event):
        self.Destroy()

class WeatherDialog(wx.Frame):
    def __init__(self):
        super().__init__(None, title="PolskaPogoda", size=(400, 200))
        panel = wx.Panel(self)

        sizer = wx.BoxSizer(wx.VERTICAL)

        self.cityBox = wx.ComboBox(panel, style=wx.CB_DROPDOWN)
        sizer.Add(wx.StaticText(panel, label="Wybierz miasto:"), 0, wx.ALL, 5)
        sizer.Add(self.cityBox, 0, wx.ALL | wx.EXPAND, 5)

        button = wx.Button(panel, label="Pokaż pogodę")
        button.Bind(wx.EVT_BUTTON, self.onShowWeather)
        sizer.Add(button, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        panel.SetSizer(sizer)
        self.Centre()
        self.Show()

        self.loadCities()
        wx.CallAfter(self.cityBox.SetFocus)

    def loadCities(self):
        try:
            url = "https://danepubliczne.imgw.pl/api/data/synop"
            response = urllib.request.urlopen(url)
            data = json.loads(response.read())

            filtered = [d for d in data if "platforma" not in d["stacja"].lower()]
            self.weatherData = {d["stacja"].lower(): d for d in filtered}

            cityNames = sorted(
                [d["stacja"] for d in filtered],
                key=locale.strxfrm
            )

            self.cityBox.AppendItems(cityNames)
            if cityNames:
                self.cityBox.SetValue(cityNames[0])

        except Exception:
            core.callLater(0, WeatherResultDialog, "Nie udało się pobrać danych pogodowych.")
            self.Close()

    def onShowWeather(self, event):
        selection = self.cityBox.GetValue().lower()
        self.showWeatherForCity(selection)

    def showWeatherForCity(self, cityName):
        entry = self.weatherData.get(cityName)
        if not entry:
            core.callLater(0, WeatherResultDialog, "Nie znaleziono danych dla tego miasta.")
            return

        temp = entry.get("temperatura", "brak")
        wind = entry.get("predkosc_wiatru", "brak")
        wind_dir = entry.get("kierunek_wiatru", "brak")
        humid = entry.get("wilgotnosc_wzgledna", "brak")
        rain = entry.get("suma_opadu", "brak")

        resultText = (
            f"Pogoda dla: {entry['stacja']}.\n"
            f"Temperatura: {temp}°C.\n"
            f"Wilgotność: {humid} procent.\n"
            f"Wiatr: {wind} km/h, kierunek: {wind_dir}°.\n"
            f"Opady: {rain} mm."
        )

        core.callLater(0, WeatherResultDialog, resultText)
        self.Close()

class GlobalPlugin(globalPluginHandler.GlobalPlugin):
    __gestures = {
        "kb:NVDA+control+w": "showWeatherDialog",
        "kb:NVDA+control+shift+w": "showQuickWeather"
    }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        gui.settingsDialogs.NVDASettingsDialog.categoryClasses.append(PolskaPogodaSettingsPanel)

    def terminate(self):
        gui.settingsDialogs.NVDASettingsDialog.categoryClasses.remove(PolskaPogodaSettingsPanel)

    @script(
        description="Otwiera pogodę IMGW dla Polski",
        category=ADDON_SUMMARY
    )
    def script_showWeatherDialog(self, gesture):
        core.callLater(0, WeatherDialog)

    @script(
        description="Pokazuje pogodę z domyślnego miasta",
        category=ADDON_SUMMARY
    )
    def script_showQuickWeather(self, gesture):
        city = config.conf["polskapogoda"]["defaultCity"].lower()
        if not city:
            core.callLater(0, WeatherResultDialog, "Nie wybrano domyślnego miasta w ustawieniach.")
            return

        try:
            url = "https://danepubliczne.imgw.pl/api/data/synop"
            response = urllib.request.urlopen(url)
            data = json.loads(response.read())
            entries = {d["stacja"].lower(): d for d in data}
            entry = entries.get(city)

            if not entry:
                core.callLater(0, WeatherResultDialog, f"Nie znaleziono miasta {city}.")
                return

            temp = entry.get("temperatura", "brak")
            wind = entry.get("predkosc_wiatru", "brak")
            wind_dir = entry.get("kierunek_wiatru", "brak")
            humid = entry.get("wilgotnosc_wzgledna", "brak")
            rain = entry.get("suma_opadu", "brak")

            resultText = (
                f"Pogoda dla: {entry['stacja']}.\n"
                f"Temperatura: {temp}°C.\n"
                f"Wilgotność: {humid} procent.\n"
                f"Wiatr: {wind} km/h, kierunek: {wind_dir}°.\n"
                f"Opady: {rain} mm."
            )
            core.callLater(0, WeatherResultDialog, resultText)
        except:
            core.callLater(0, WeatherResultDialog, "Błąd pobierania danych.")
